@@ -62,15 +62,22 @@ class Artists(Gtk.Box):
     '''Artists tab in notebook.'''
 
     title = _('Artists')
+    first_run = True
 
     def __init__(self, app):
         super().__init__(orientation=Gtk.Orientation.VERTICAL)
         self.app = app
 
-        app = self.app
         self.buttonbox = Gtk.Box()
         self.pack_start(self.buttonbox, False, False, 0)
 
+    def first(self):
+        if not self.first_run:
+            return
+        self.first_run = False
+
+        app = self.app
+
         home_button = Gtk.Button(_('Artists'))
         home_button.connect('clicked', self.on_home_button_clicked)
         self.buttonbox.pack_start(home_button, False, False, 0)
@@ -367,51 +374,51 @@ class Artists(Gtk.Box):
 
         # load current favorite artists list
         self.load_fav_artists()
-
-    def first(self):
-        pass
-
-    def after_init(self):
+        self.show_all()
         self.buttonbox.hide()
 
+    def after_init(self):
+        pass
+
     def do_destroy(self):
-        self.dump_fav_artists()
+        if not self.first_run:
+            self.dump_fav_artists()
 
     def on_cate_changed(self, *args):
         self.append_artists(init=True)
 
     def append_artists(self, init=False):
+        def on_append_artists(info, error=None):
+            artists, self.artists_total = info
+            if error or not self.artists_total or not artists:
+                return
+            for artist in artists:
+                _info = ' '.join([artist['music_num'], _(' songs'), ])
+                tree_iter = self.artists_liststore.append([
+                    self.app.theme['anonymous'],
+                    Widgets.unescape(artist['name']),
+                    int(artist['id']),
+                    _info,
+                    Widgets.set_tooltip(artist['name'], _info),
+                    ])
+                Net.update_artist_logo(
+                        self.artists_liststore, tree_iter, 0, artist['pic'])
+
         if init:
             self.artists_liststore.clear()
             self.artists_page = 0
             self.artists_win.get_vadjustment().set_value(0)
         selection = self.cate_treeview.get_selection()
-        result = selection.get_selected()
-        if result is None or len(result) != 2:
+        selected = selection.get_selected()
+        if not selected:
             return
-        model, _iter = result
+        model, _iter = selected
         pref_index = self.pref_combo.get_active()
         catid = model[_iter][1]
         prefix = self.pref_liststore[pref_index][1]
-        # TODO: use async_call()
-        artists, self.artists_total = Net.get_artists(
-                catid, self.artists_page, prefix)
-        if self.artists_total == 0:
-            return
-
-        i = len(self.artists_liststore)
-        for artist in artists:
-            _info = ' '.join([artist['music_num'], _(' songs'), ])
-            self.artists_liststore.append([
-                self.app.theme['anonymous'],
-                Widgets.unescape(artist['name']),
-                int(artist['id']), 
-                _info,
-                Widgets.set_tooltip(artist['name'], _info),
-                ])
-            Net.update_artist_logo(
-                    self.artists_liststore, i, 0, artist['pic'])
-            i += 1
+        Net.async_call(
+                Net.get_artists, on_append_artists, catid,
+                self.artists_page, prefix)
 
     def on_artists_iconview_item_activated(self, iconview, path):
         model = iconview.get_model()
@@ -431,9 +438,10 @@ class Artists(Gtk.Box):
             self.artists_page += 1
             self.append_artists()
 
-
-    # open API other tabs can use.
+    # open API
     def show_artist(self, artist, artistid):
+        '''Show artist tab'''
+        self.first()
         self.curr_artist_name = artist
         self.curr_artist_id = artistid
         self.notebook.set_current_page(1)
@@ -463,6 +471,7 @@ class Artists(Gtk.Box):
             self.artist_songs_button.set_active(True)
 
     def show_artist_songs(self):
+        '''Show all songs of this artist'''
         self.album_control_box.hide()
         self.artist_mv_control_box.hide()
         self.artist_control_box.show_all()
@@ -474,7 +483,7 @@ class Artists(Gtk.Box):
     def append_artist_songs(self, init=False):
         def _append_artist_songs(songs_args, error=None):
             songs, self.artist_songs_total = songs_args
-            if self.artist_songs_total == 0:
+            if error or self.artist_songs_total == 0:
                 return
             for song in songs:
                 self.artist_songs_liststore.append([
@@ -510,11 +519,10 @@ class Artists(Gtk.Box):
     def append_artist_albums(self, init=False):
         def _append_artist_albums(albums_args, error=None):
             albums, self.artist_albums_total = albums_args
-            if self.artist_albums_total == 0:
+            if error or self.artist_albums_total == 0:
                 return
-            i = len(self.artist_albums_liststore)
             for album in albums:
-                self.artist_albums_liststore.append([
+                tree_iter = self.artist_albums_liststore.append([
                     self.app.theme['anonymous'],
                     Widgets.unescape(album['name']),
                     int(album['albumid']),
@@ -523,8 +531,8 @@ class Artists(Gtk.Box):
                     Widgets.set_tooltip(album['name'], album['info']),
                     ])
                 Net.update_album_covers(
-                        self.artist_albums_liststore, i, 0, album['pic'])
-                i += 1
+                        self.artist_albums_liststore,
+                        tree_iter, 0, album['pic'])
             self.artist_albums_page += 1
             if self.artist_albums_page < self.artist_albums_total - 1:
                 self.append_artist_albums()
@@ -548,11 +556,10 @@ class Artists(Gtk.Box):
     def append_artist_mv(self, init=False):
         def _append_artist_mv(mv_args, error=None):
             mvs, self.artist_mv_total = mv_args
-            if self.artist_mv_total == 0:
+            if error or self.artist_mv_total == 0:
                 return
-            i = len(self.artist_mv_liststore)
             for mv in mvs:
-                self.artist_mv_liststore.append([
+                tree_iter = self.artist_mv_liststore.append([
                     self.app.theme['anonymous'],
                     Widgets.unescape(mv['name']),
                     Widgets.unescape(mv['artist']),
@@ -563,8 +570,7 @@ class Artists(Gtk.Box):
                     Widgets.set_tooltip(mv['name'], mv['artist']),
                     ])
                 Net.update_mv_image(
-                        self.artist_mv_liststore, i, 0, mv['pic'])
-                i += 1
+                        self.artist_mv_liststore, tree_iter, 0, mv['pic'])
             self.artist_mv_page += 1
             if self.artist_mv_page < self.artist_mv_total - 1:
                 self.append_artist_mv()
@@ -586,14 +592,14 @@ class Artists(Gtk.Box):
         self.append_artist_similar(init=True)
 
     def append_artist_similar(self, init=False):
+        self.first()
         def _append_artist_similar(similar_args, error=None):
             artists, self.artist_similar_total = similar_args
-            if self.artist_similar_total == 0:
+            if error or not self.artist_similar_total:
                 return
-            i = len(self.artist_similar_liststore)
             for artist in artists:
                 _info = ''.join([artist['songnum'], _(' songs'), ])
-                self.artist_similar_liststore.append([
+                tree_iter = self.artist_similar_liststore.append([
                     self.app.theme['anonymous'],
                     Widgets.unescape(artist['name']),
                     int(artist['id']),
@@ -601,8 +607,8 @@ class Artists(Gtk.Box):
                     Widgets.set_tooltip(artist['name'], _info),
                     ])
                 Net.update_artist_logo(
-                        self.artist_similar_liststore, i, 0, artist['pic'])
-                i += 1
+                        self.artist_similar_liststore,
+                        tree_iter, 0, artist['pic'])
             self.artist_similar_page += 1
             if self.artist_similar_page < self.artist_similar_total - 1:
                 self.append_artist_similar()
@@ -625,7 +631,9 @@ class Artists(Gtk.Box):
 
     def append_artist_info(self):
         def _append_artist_info(info, error=None):
-            if info and info['pic'] and len(info['pic']) > 0:
+            if error or not info:
+                return
+            if info.get('pic', None):
                 self.artist_info_pic.set_from_file(info['pic'])
             self.artist_info_name.set(info, 'name')
             self.artist_info_birthday.set(info, 'birthday')
@@ -669,6 +677,8 @@ class Artists(Gtk.Box):
 
     # open API
     def show_album(self, album, albumid, artist, artistid):
+        '''Show album information, including songs'''
+        self.first()
         self.curr_album_name = album
         self.curr_album_id = albumid
         self.curr_artist_name = artist
@@ -684,7 +694,7 @@ class Artists(Gtk.Box):
     
     def append_album_songs(self):
         def _append_album_songs(songs, error=None):
-            if not songs:
+            if error or not songs:
                 return
             for song in songs:
                 self.album_songs_liststore.append([
@@ -706,15 +716,14 @@ class Artists(Gtk.Box):
     # Signal handlers for favorite artists tab
     def add_to_fav_artists(self, artist_id, init=False):
         def _append_fav_artist(info, error=None):
-            if info and info['pic'] and len(info['pic']) > 0:
+            if error or not info:
+                return
+            if info.get('pic', None):
                 pix = GdkPixbuf.Pixbuf.new_from_file_at_size(
                         info['pic'], 100, 100)
             else:
                 pix = self.app.theme['anonymous']
-            if 'info' in info:
-                tip = Widgets.escape(info['info'])
-            else:
-                tip = ''
+            tip = Widgets.escape(info.get('info', ''))
             self.fav_artists_liststore.append(
                     [pix, info['name'], artist_id, tip])
 
@@ -730,7 +739,7 @@ class Artists(Gtk.Box):
                 return
 
     def check_artist_favorited(self, artist_id):
-        '''Check if this artist in favorite list.'''
+        '''Check whether this artist is in favorite list.'''
         for row in self.fav_artists_liststore:
             if artist_id == row[2]:
                 return True
