@@ -10,9 +10,11 @@ import math
 import os
 import sys
 import threading
+import time
 from urllib.error import URLError
 from urllib import parse
 from urllib import request
+
 from gi.repository import GdkPixbuf
 from gi.repository import GLib
 from gi.repository import GObject
@@ -28,14 +30,15 @@ QUKU_SONG = 'http://nplserver.kuwo.cn/pl.svc?'
 SEARCH = 'http://search.kuwo.cn/r.s?'
 SONG = 'http://antiserver.kuwo.cn/anti.s?'
 
-CHUNK = 2 ** 14
-CHUNK_TO_PLAY = 2 ** 21      # 2M
-CHUNK_APE_TO_PLAY = 2 ** 23  # 8M
-CHUNK_MV_TO_PLAY = 2 ** 23   # 8M
-MAXTIMES = 3
-TIMEOUT = 30
-SONG_NUM = 100
-ICON_NUM = 50
+CHUNK = 16384                # 2**14, 16k, chunk size for file downloading 
+CHUNK_TO_PLAY = 2097152      # 2**21, 2M, min size to emit can-play signal
+CHUNK_APE_TO_PLAY = 8388608  # 2**23, 8M
+CHUNK_MV_TO_PLAY = 8388608   # 2**23, 8M
+MAXTIMES = 3                 # time to retry http connections
+TIMEOUT = 30                 # HTTP connection timeout
+SONG_NUM = 100               # num of songs in each request
+ICON_NUM = 50                # num of icons in each request
+CACHE_TIMEOUT = 1209600      # 14 days in seconds
 
 # Using weak reference to cache song list in TopList and Radio.
 class Dict(dict):
@@ -52,7 +55,6 @@ except ImportError as e:
         from plyvel import DB as LevelDB
         ldb_imported = True
     except ImportError as e:
-        print('Warning: No leveldb/plyvel module was found')
         ldb_imported = False
 
 ldb = None
@@ -68,8 +70,6 @@ if ldb_imported:
             ldb_put = ldb.Put
     except Exception as e:
         ldb_imported = False
-        print(e)
-        sys.exit(1)
 
 def empty_func(*args, **kwds):
     pass
@@ -87,6 +87,7 @@ def async_call(func, func_done, *args):
         GLib.idle_add(lambda: func_done(result, error))
 
     thread = threading.Thread(target=do_call, args=args)
+    thread.daemon = True
     thread.start()
 
 def hash_byte(_str):
@@ -98,27 +99,28 @@ def hash_str(_str):
 def urlopen(_url, use_cache=True, retries=MAXTIMES):
     # set host port from 81 to 80, to fix image problem
     url = _url.replace(':81', '')
-    print('Net.urlopen:', url, ', use_cache:', use_cache)
-    # hash the url to accelerate string compare speed in db.
-    key = hash_byte(url)
     if use_cache and ldb_imported:
+        key = hash_byte(url)
         try:
             content = ldb_get(key)
-            if content:
-                return content
+            if content and len(content) > 10:
+                try:
+                    timestamp = int(content[:10].decode())
+                    if (time.time() - timestamp) < CACHE_TIMEOUT:
+                        return content[10:]
+                except (ValueError, UnicodeDecodeError):
+                    pass
         except KeyError:
             pass
-    retried = 0
-    while retried < retries:
+    for i in range(retries):
         try:
             req = request.urlopen(url, timeout=TIMEOUT)
             req_content = req.read()
             if use_cache and ldb_imported:
-                ldb_put(key, req_content)
+                ldb_put(key, str(int(time.time())).encode() + req_content)
             return req_content
         except URLError as e:
-            print('Error: Net.urlopen', e, 'url:', url)
-            retried += 1
+            pass
     return None
 
 def get_nodes(nid, page):
@@ -132,14 +134,12 @@ def get_nodes(nid, page):
         '&node=',
         str(nid),
         ])
-    #print('get_nodes()', url)
     req_content = urlopen(url)
     if not req_content:
         return (None, 0)
     try:
         nodes_wrap = json.loads(req_content.decode())
     except ValueError as e:
-        print('Error: Net.get_nodes:', e, 'with url:', url)
         return (None, 0)
     nodes = nodes_wrap['child']
     pages = math.ceil(int(nodes_wrap['total']) / ICON_NUM)
@@ -170,33 +170,30 @@ def get_album(albumid):
         'stype=albuminfo&albumid=',
         str(albumid),
         ])
-    #print('get_album():', url)
     req_content = urlopen(url)
     if not req_content:
         return None
     try:
         songs_wrap = Utils.json_loads_single(req_content.decode())
     except ValueError as e:
-        print('Error: Net.get_album()', e, 'with url:', url)
         return None
     songs = songs_wrap['musiclist']
     return songs
 
-def update_liststore_image(liststore, path, col, url):
+def update_liststore_image(liststore, tree_iter, col, url):
     def _update_image(filepath, error=None):
         if not filepath or error:
             return
         try:
             pix = GdkPixbuf.Pixbuf.new_from_file_at_size(filepath, 100, 100)
-            liststore[path][col] = pix
+            liststore[liststore.get_path(tree_iter)][col] = pix
         except GLib.GError as e:
-            print('Error: Net.update_liststore_image:', e, 
-                  'with filepath:', filepath, 'url:', url)
+            pass
     if len(url) < 10:
         return
     async_call(get_image, _update_image, url)
 
-def update_album_covers(liststore, path, col, _url):
+def update_album_covers(liststore, tree_iter, col, _url):
     url = _url.strip()
     if url and len(url) == 0:
         return None
@@ -205,10 +202,9 @@ def update_album_covers(liststore, path, col, _url):
         'star/albumcover/',
         url,
         ])
-    #print('update_album_covers()', url)
-    update_liststore_image(liststore, path, col, url)
+    update_liststore_image(liststore, tree_iter, col, url)
 
-def update_mv_image(liststore, path, col, _url):
+def update_mv_image(liststore, tree_iter, col, _url):
     url = _url.strip()
     if len(url) == 0:
         return None
@@ -217,7 +213,7 @@ def update_mv_image(liststore, path, col, _url):
         'wmvpic/',
         url,
         ])
-    update_liststore_image(liststore, path, col, url)
+    update_liststore_image(liststore, tree_iter, col, url)
 
 def get_toplist_songs(nid):
     # no need to use pn, because toplist contains very few songs
@@ -228,7 +224,6 @@ def get_toplist_songs(nid):
         '&id=',
         str(nid),
         ])
-    #print('get toplist songs(), url:', url)
     if url not in req_cache:
         req_content = urlopen(url, use_cache=False)
         if not req_content:
@@ -237,7 +232,6 @@ def get_toplist_songs(nid):
     try:
         songs_wrap = json.loads(req_cache[url].decode())
     except ValueError as e:
-        print('Error: Net.get_toplist_songs:', e, 'with url:', url)
         return None
     return songs_wrap['musiclist']
 
@@ -253,21 +247,19 @@ def get_artists(catid, page, prefix):
         ])
     if len(prefix) > 0:
         url = url + '&prefix=' + prefix
-    #print('Net.get_artists(), url:', url)
     req_content = urlopen(url)
     if not req_content:
         return (None, 0)
     try:
         artists_wrap = Utils.json_loads_single(req_content.decode())
     except ValueError as e:
-        print('Error: Net.get_artists:', e, 'with url:', url)
         return (None, 0)
     pages = int(artists_wrap['total'])
     artists = artists_wrap['artistlist']
     return (artists, pages)
 
-def update_toplist_node_logo(liststore, path, col, url):
-    update_liststore_image(liststore, path, col, url)
+def update_toplist_node_logo(liststore, tree_iter, col, url):
+    update_liststore_image(liststore, tree_iter, col, url)
 
 def get_artist_pic_url(pic_path):
     if len(pic_path) < 5:
@@ -277,11 +269,11 @@ def get_artist_pic_url(pic_path):
     url = ''.join([IMG_CDN, 'star/starheads/', pic_path, ])
     return url
 
-def update_artist_logo(liststore, path, col, pic_path):
+def update_artist_logo(liststore, tree_iter, col, pic_path):
     url = get_artist_pic_url(pic_path)
     if not url:
         return
-    update_liststore_image(liststore, path, col, url)
+    update_liststore_image(liststore, tree_iter, col, url)
 
 def get_artist_info(artistid, artist=None):
     '''Get artist info, if cached, just return it.
@@ -300,14 +292,12 @@ def get_artist_info(artistid, artist=None):
             'stype=artistinfo&artistid=', 
             str(artistid),
             ])
-    #print('Net.get_artist_info, url:', url)
     req_content = urlopen(url)
     if not req_content:
         return None
     try:
         info = Utils.json_loads_single(req_content.decode())
     except ValueError as e:
-        print('Error: Net.get_artist_info:', e, 'with url:', url)
         return None
     # set logo size to 100x100
     pic_path = info['pic']
@@ -328,14 +318,12 @@ def get_artist_songs(artist, page):
         '&primitive=0&rformat=json&encoding=UTF8&artist=',
         artist,
         ])
-    #print('Net.get_artist_songs()', url)
     req_content = urlopen(url)
     if not req_content:
         return (None, 0)
     try:
         songs_wrap = Utils.json_loads_single(req_content.decode())
     except ValueError as e:
-        print('Error: Net.get_artist_songs:', e, 'with url:', url)
         return (None, 0)
     songs = songs_wrap['abslist']
     pages = math.ceil(int(songs_wrap['TOTAL']) / SONG_NUM)
@@ -351,14 +339,12 @@ def get_artist_songs_by_id(artistid, page):
         '&pn=',
         str(page),
         ])
-    #print('Net.get_artist_songs_by_id()', url)
     req_content = urlopen(url)
     if not req_content:
         return (None, 0)
     try:
         songs_wrap = Utils.json_loads_single(req_content.decode())
     except ValueError as e:
-        print('Error: Net.get-artist_songs_by_id:', e, 'with url:', url)
         return (None, 0)
     songs = songs_wrap['musiclist']
     pages = math.ceil(int(songs_wrap['total']) / SONG_NUM)
@@ -374,14 +360,12 @@ def get_artist_albums(artistid, page):
         '&pn=',
         str(page),
         ])
-    #print('Net.get_artist_albums(), url:', url)
     req_content = urlopen(url)
     if not req_content:
         return (None, 0)
     try:
         albums_wrap = Utils.json_loads_single(req_content.decode())
     except ValueError as e:
-        print('Error:', e)
         return (None, 0)
     albums = albums_wrap['albumlist']
     pages = math.ceil(int(albums_wrap['total']) / ICON_NUM)
@@ -397,14 +381,12 @@ def get_artist_mv(artistid, page):
         '&pn=',
         str(page),
         ])
-    #print('Net.get_artist_mv(), url:', url)
     req_content = urlopen(url)
     if not req_content:
         return (None, 0)
     try:
         mvs_wrap = Utils.json_loads_single(req_content.decode())
     except ValueError as e:
-        print('Error: Net.get_artist_mv:', e, 'with url:', url)
         return (None, 0)
     mvs = mvs_wrap['mvlist']
     pages = math.ceil(int(mvs_wrap['total']) / ICON_NUM)
@@ -421,14 +403,12 @@ def get_artist_similar(artistid, page):
         '&artistid=',
         str(artistid),
         ])
-    #print('Net.get_artist_similar(), url:', url)
     req_content = urlopen(url)
     if not req_content:
         return (None, 0)
     try:
        artists_wrap = Utils.json_loads_single(req_content.decode())
     except ValueError as e:
-        print('Error: Net.get_artist_similar:', e, 'with url:', url)
         return (None, 0)
     artists = artists_wrap['artistlist']
     pages = math.ceil(int(artists_wrap['total']) / ICON_NUM)
@@ -450,7 +430,6 @@ def get_lrc(song):
     def _parse_lrc():
         url = ('http://newlyric.kuwo.cn/newlyric.lrc?' + 
                 Utils.encode_lrc_url(rid))
-        #print('Net._parse_lrc(), url:', url)
         req_content = urlopen(url, use_cache=False, retries=8)
         if not req_content:
             return None
@@ -458,7 +437,6 @@ def get_lrc(song):
             lrc = Utils.decode_lrc_content(req_content)
             return lrc
         except Exception as e:
-            print('Error: Net.get_lrc:', e, 'with url:', url)
             return None
 
     rid = str(song['rid'])
@@ -473,7 +451,7 @@ def get_lrc(song):
             with open(lrc_path, 'w') as fh:
                 fh.write(lrc)
         except FileNotFoundError as e:
-            print('Error: Net.get_lrc(): ', e)
+            pass
     return lrc
 
 def get_recommend_lists(artist):
@@ -523,7 +501,6 @@ def search_songs(keyword, page):
         '&pn=',
         str(page),
         ])
-    #print('search songs:', url)
     if url not in req_cache:
         req_content = urlopen(url, use_cache=False)
         if not req_content:
@@ -532,7 +509,6 @@ def search_songs(keyword, page):
     try:
         songs_wrap = Utils.json_loads_single(req_cache[url].decode())
     except ValueError as e:
-        print('Error: Net.search_song:', e, 'with url:', url)
         return (None, 0, 0)
     hit = int(songs_wrap['TOTAL'])
     pages = math.ceil(hit / SONG_NUM)
@@ -550,7 +526,6 @@ def search_artists(keyword, page):
         '&itemset=newkm&rformat=json&encoding=utf8&all=',
         parse.quote(keyword),
         ])
-    #print('Net.search_artists(), ', url)
     if url not in req_cache:
         req_content = urlopen(url, use_cache=False)
         if not req_content:
@@ -559,7 +534,6 @@ def search_artists(keyword, page):
     try:
         artists_wrap = Utils.json_loads_single(req_cache[url].decode())
     except ValueError as e:
-        print('Error: Net.search_artists():', e, 'with url:', url)
         return (None, 0, 0)
     hit = int(artists_wrap['TOTAL'])
     pages = math.ceil(hit / SONG_NUM)
@@ -577,7 +551,6 @@ def search_albums(keyword, page):
         '&itemset=newkm&rformat=json&encoding=utf8&all=',
         parse.quote(keyword),
         ])
-    #print('search_albums:', url)
     if url not in req_cache:
         req_content = urlopen(url, use_cache=False)
         if not req_content:
@@ -586,7 +559,6 @@ def search_albums(keyword, page):
     try:
         albums_wrap = Utils.json_loads_single(req_cache[url].decode())
     except ValueError  as e:
-        print('Error: Net.search_albums():', e, 'with url:', url)
         return (None, 0, 0)
     hit = int(albums_wrap['total'])
     pages = math.ceil(hit / ICON_NUM)
@@ -602,14 +574,12 @@ def get_index_nodes(nid):
         '&node=',
         str(nid),
         ])
-    #print('get_index_nodes():', url)
     req_content = urlopen(url)
     if not req_content:
         return None
     try:
         nodes_wrap = json.loads(req_content.decode())
     except ValueError as e:
-        print('Error: Net.get_index_nodes():', e, 'with url:', url)
         return None
     return nodes_wrap
 
@@ -670,7 +640,6 @@ def get_themes_songs(nid, page):
         '&pn=',
         str(page),
         ])
-    #print('Net.get themes songs, url:', url)
     if url not in req_cache:
         req_content = urlopen(url, use_cache=False)
         if not req_content:
@@ -679,7 +648,6 @@ def get_themes_songs(nid, page):
     try:
         songs_wrap = json.loads(req_cache[url].decode())
     except ValueError as e:
-        print('Error: Net.get_themes_songs():', e, 'with url:', url)
         return (None, 0)
     pages = math.ceil(int(songs_wrap['total']) / SONG_NUM)
     return (songs_wrap['musiclist'], pages)
@@ -694,14 +662,12 @@ def get_mv_songs(pid, page):
         '&encode=utf-8&keyset=mvpl&pid=',
         str(pid),
         ])
-    #print('Net.get_mv_songs(), url:', url)
     req_content = urlopen(url)
     if not req_content:
         return (None, 0)
     try:
         songs_wrap = json.loads(req_content.decode())
     except ValueError as e:
-        print('Error: Net.get_mv_songs():', e, 'with url:', url)
         return (None, 0)
     songs = songs_wrap['musiclist']
     pages = math.ceil(int(songs_wrap['total']) / ICON_NUM)
@@ -725,36 +691,6 @@ def get_radio_songs(nid, offset):
     songs = Utils.parse_radio_songs(req_content.decode('gbk'))
     return songs
 
-def _get_song_link_raw(song, conf, use_mv=False):
-    if use_mv:
-        ext = 'mkv|mp4' if conf['use-mkv'] else 'mp4'
-    else:
-        ext = 'ape|mp3' if conf['use-ape'] else 'mp3'
-    url = ''.join([
-        SONG,
-        'response=url&type=convert_url&format=',
-        ext,
-        '&rid=MUSIC_',
-        str(song['rid']),
-        ])
-    #print('Net._get_song_link_raw(), url', url)
-    req_content = urlopen(url)
-    if not req_content:
-        return None
-    song_link = req_content.decode()
-    if len(song_link) < 20:
-        return None
-    song_list = song_link.split('/')
-    song_link = '/'.join(song_list[:3] + song_list[5:])
-    song_name = os.path.split(song_link)[1]
-    if use_mv:
-        song_path = os.path.join(conf['mv-dir'], song_name)
-    else:
-        song_path = os.path.join(conf['song-dir'], song_name)
-    if os.path.exists(song_path):
-        return (True, song_path)
-    return (song_link, song_path)
-
 def get_song_link(song, conf, use_mv=False):
     '''song is song_info dict.
 
@@ -762,9 +698,10 @@ def get_song_link(song, conf, use_mv=False):
     conf['song-dir'] and song['mv-dir'].
     use_mv, default is False, which will get mp3 link.
     Return:
-     @song_link: if is True, local song exists
-                 if is False, no available song_link
-                 if is str, this link can be downloaded.
+     @cached : if is True, local song exists
+               if is False, no available song_link
+     @song_link: music source link; if local song exists or failed to get
+              music source link, returns ''
      @song_path: target abs-path this song will be cached.
     '''
     if use_mv:
@@ -778,7 +715,6 @@ def get_song_link(song, conf, use_mv=False):
         '&rid=MUSIC_',
         str(song['rid']),
         ])
-    #print('Net.get_song_link(), url', url)
     song_name = (''.join([
         song['artist'],
         '-',
@@ -791,21 +727,21 @@ def get_song_link(song, conf, use_mv=False):
     else:
         song_path = os.path.join(conf['song-dir'], song_name)
     if os.path.exists(song_path):
-        # if song/MV exists, just return it
-        return (True, song_path)
+        return (True, '', song_path)
     req_content = urlopen(url)
     if not req_content:
-        return (False, song_path)
+        return (False, '', song_path)
     song_link = req_content.decode()
     if len(song_link) < 20:
-        return (False, song_path)
+        return (False, '', song_path)
     song_list = song_link.split('/')
     song_link = '/'.join(song_list[:3] + song_list[5:])
+    # update song path
     song_path = ''.join([os.path.splitext(song_path)[0],
                          os.path.splitext(song_link)[1]])
     if os.path.exists(song_path):
-        return (True, song_path)
-    return (song_link, song_path)
+        return (True, '', song_path)
+    return (False, song_link, song_path)
 
 
 class AsyncSong(GObject.GObject):
@@ -823,13 +759,17 @@ class AsyncSong(GObject.GObject):
     __gsignals__ = {
             'can-play': (GObject.SIGNAL_RUN_LAST, 
                 # sogn_path
-                GObject.TYPE_NONE, (str, str)),
+                GObject.TYPE_NONE, (str, )),
             'chunk-received': (GObject.SIGNAL_RUN_LAST,
                 # percent
-                GObject.TYPE_NONE, (int, )),
+                GObject.TYPE_NONE, (GObject.TYPE_DOUBLE, )),
             'downloaded': (GObject.SIGNAL_RUN_LAST, 
                 # song_path
-                GObject.TYPE_NONE, (str, ))
+                GObject.TYPE_NONE, (str, )),
+            'disk-error': (GObject.SIGNAL_RUN_LAST,
+                GObject.TYPE_NONE, (str, )),
+            'network-error': (GObject.SIGNAL_RUN_LAST,
+                GObject.TYPE_NONE, (str, )),
             }
 
     def __init__(self, app):
@@ -847,32 +787,29 @@ class AsyncSong(GObject.GObject):
         like this:
         response=url&type=convert_url&format=ape|mp3&rid=MUSIC_3312608
         '''
-        print('AsyncSong.get_song():', song, 'use_mv: ', use_mv)
         async_call(self._download_song, empty_func, song, use_mv)
 
     def _download_song(self, song, use_mv):
-        song_link, song_path = get_song_link(
+        cached, song_link, song_path = get_song_link(
                 song, self.app.conf, use_mv=use_mv)
-        print('Async_song.download_song:', song_link, song_path)
 
-        #print('Net.AsyncSong._download_song() song link:', song_link)
+        # check song already cached 
+        if cached:
+            self.emit('can-play', song_path)
+            self.emit('downloaded', song_path)
+            return
+
+        # this song has no link to download
+        if not song_link:
+            self.emit('network-error', song_link)
+            return
+
         chunk_to_play = CHUNK_TO_PLAY
         if self.app.conf['use-ape']:
             chunk_to_play = CHUNK_APE_TO_PLAY
         if use_mv:
             chunk_to_play = CHUNK_MV_TO_PLAY
 
-        # this song has no link to download
-        if song_link is False:
-            self.emit('can-play', song_path, 'URLError')
-            return
-
-        # check lock song exists
-        if song_link is True:
-            self.emit('can-play', song_path, 'OK')
-            self.emit('downloaded', song_path)
-            return
-
         for retried in range(MAXTIMES):
             try:
                 req = request.urlopen(song_link)
@@ -889,14 +826,13 @@ class AsyncSong(GObject.GObject):
                         return
                     chunk = req.read(CHUNK)
                     received_size += len(chunk)
-                    percent = int(received_size/content_length * 100)
-                    print(percent)
+                    percent = received_size / content_length
                     self.emit('chunk-received', percent)
                     # this signal only emit once.
                     if ((received_size > chunk_to_play or percent > 40) and
                             not can_play_emited):
                         can_play_emited = True
-                        self.emit('can-play', song_path, 'OK')
+                        self.emit('can-play', song_path)
                     if not chunk:
                         break
                     fh.write(chunk)
@@ -906,12 +842,12 @@ class AsyncSong(GObject.GObject):
                 return
 
             except URLError as e:
-                print('AsyncSong._download_song()', e,
-                      'with song_link:', song_link)
+                pass
+            except FileNotFoundError as e:
+                self.emit('disk-error', song_path)
+                return
         if os.path.exists(song_path):
             os.remove(song_path)
-            self.emit('can-play', song_path, 'URLError')
-        else:
-            self.emit('can-play', song_path, 'FileNotFoundError')
+        self.emit('network-error', song_link)
 
 GObject.type_register(AsyncSong)
