@@ -49,7 +49,7 @@ class NormalSongTab(Gtk.ScrolledWindow):
         self.liststore = Gtk.ListStore(str, str, str, int, int, int)
 
         self.treeview = Gtk.TreeView(model=self.liststore)
-        #self.treeview.set_headers_visible(False)
+        self.selection = self.treeview.get_selection()
         self.treeview.set_search_column(0)
         self.treeview.props.headers_clickable = True
         self.treeview.props.headers_visible = True
@@ -83,18 +83,30 @@ class NormalSongTab(Gtk.ScrolledWindow):
 
         delete_cell = Gtk.CellRendererPixbuf(
                 icon_name='user-trash-symbolic')
-        delete_col = Widgets.TreeViewColumnIcon(_('Delete'), delete_cell)
-        self.treeview.append_column(delete_col)
-        self.connect('key-press-event', self.on_key_pressed)
+        self.delete_col = Widgets.TreeViewColumnIcon(
+                _('Delete'), delete_cell)
+        self.treeview.append_column(self.delete_col)
+        self.treeview.connect(
+                'key-press-event', self.on_treeview_key_pressed)
+        self.treeview.connect(
+                'button-press-event', self.on_treeview_button_pressed)
         
-    def on_key_pressed(self, widget, event):
+    def on_treeview_key_pressed(self, treeview, event):
         if event.keyval == Gdk.KEY_Delete:
-            selection = self.treeview.get_selection()
-            model, paths = selection.get_selected_rows()
+            model, paths = self.selection.get_selected_rows()
             # paths needs to be reversed, or else an IndexError throwed.
             for path in reversed(paths):
                 model.remove(model[path].iter)
 
+    def on_treeview_button_pressed(self, treeview, event):
+        path_info = treeview.get_path_at_pos(event.x, event.y)
+        if not path_info:
+            return
+        path, column, cell_x, cell_y = path_info
+        if column != self.delete_col:
+            return
+        self.liststore.remove(self.liststore[path].iter)
+
     def on_treeview_row_activated(self, treeview, path, column):
         model = treeview.get_model()
         index = treeview.get_columns().index(column)
@@ -105,8 +117,6 @@ class NormalSongTab(Gtk.ScrolledWindow):
             self.app.search.search_artist(song['artist'])
         elif index == 2:
             self.app.search.search_album(song['album'])
-        elif index == 3:
-            model.remove(model[path].iter)
 
     def on_drag_data_get(self, treeview, drag_context, sel_data, info, 
                          time):
@@ -139,6 +149,77 @@ class NormalSongTab(Gtk.ScrolledWindow):
             model.remove(_iter)
 
 
+class ExportDialog(Gtk.Dialog):
+
+    def __init__(self, parent, liststore):
+        super().__init__(
+                _('Export Songs'), parent.app.window, Gtk.DialogFlags.MODAL,
+                (Gtk.STOCK_CLOSE, Gtk.ResponseType.OK,))
+        self.parent = parent
+        self.liststore = liststore
+        self.app = parent.app
+
+        self.set_border_width(15)
+
+        box = self.get_content_area()
+        box.set_size_request(540, 260)
+        box.set_spacing = 5
+
+        folder_label = Widgets.BoldLabel(_('Choose export folder:'))
+        box.pack_start(folder_label, False, True, 2)
+
+        self.folder_chooser = Widgets.FolderChooser(self.app.window)
+        self.folder_chooser.props.margin_left = 20
+        box.pack_start(self.folder_chooser, False, True, 0)
+
+        self.with_lrc = Gtk.CheckButton(_('With lyrics'))
+        self.with_lrc.set_tooltip_text(_('Export lyrics to the same folder'))
+        self.with_lrc.props.margin_top = 20
+        box.pack_start(self.with_lrc, False, False, 0)
+
+        export_box = Gtk.Box(spacing=5)
+        export_box.props.margin_top = 20
+        box.pack_start(export_box, False, True, 0)
+
+        self.export_prog = Gtk.ProgressBar()
+        self.export_prog.props.show_text = True
+        self.export_prog.props.text = ''
+        export_box.pack_start(self.export_prog, True, True, 0)
+
+        export_btn = Gtk.Button(_('Export'))
+        export_btn.connect('clicked', self.do_export)
+        export_box.pack_start(export_btn, False, False, 0)
+
+        infobar = Gtk.InfoBar()
+        infobar.props.margin_top = 20
+        box.pack_start(infobar, False, True, 0)
+        info_content = infobar.get_content_area()
+        info_label = Gtk.Label(_('Only cached songs will be exported'))
+        info_content.pack_start(info_label, False, False, 0)
+        box.show_all()
+
+    def do_export(self, button):
+        num_songs = len(self.liststore)
+        export_dir = self.folder_chooser.get_filename()
+        export_lrc = self.with_lrc.get_active()
+        for i, item in enumerate(self.liststore):
+            song = Widgets.song_row_to_dict(item, start=0)
+            cached, song_link, song_path = Net.get_song_link(
+                    song, self.app.conf)
+            if not cached:
+                continue
+            self.export_prog.set_fraction(i / num_songs)
+            self.export_prog.set_text(song['name'])
+            shutil.copy(song_path, export_dir)
+
+            if export_lrc:
+                lrc_path, lrc_cached = Net.get_lrc_path(song)
+                if lrc_cached:
+                    shutil.copy(lrc_path, export_dir)
+            Gdk.Window.process_all_updates()
+        self.destroy()
+
+
 class PlayList(Gtk.Box):
     '''Playlist tab in notebook.'''
 
@@ -347,12 +428,11 @@ class PlayList(Gtk.Box):
             # curr_playing contains: listname, path
             self.curr_playing = [list_name, path]
             song = Widgets.song_row_to_dict(liststore[path], start=0)
-            self.app.player.load(song)
-            return
-        liststore.append(Widgets.song_dict_to_row(song))
-        self.curr_playing = [list_name, len(liststore)-1, ]
-        self.locate_curr_song(popup_page=False)
-        if use_mv:
+        else:
+            liststore.append(Widgets.song_dict_to_row(song))
+            self.curr_playing = [list_name, len(liststore)-1, ]
+            self.locate_curr_song(popup_page=False)
+        if use_mv is True:
             self.app.player.load_mv(song)
         else:
             self.app.player.load(song)
@@ -405,11 +485,15 @@ class PlayList(Gtk.Box):
         tree = self.tabs[list_name].treeview
         Net.async_call(start, stop)
 
+    # Open API
     def cache_song(self, song):
         rid = song['rid']
         liststore = self.tabs['Caching'].liststore
         liststore.append(Widgets.song_dict_to_row(song))
+        if not self.cache_enabled:
+            self.switch_caching_daemon()
 
+    # Open API
     def cache_songs(self, songs):
         for song in songs:
             self.cache_song(song)
@@ -419,7 +503,7 @@ class PlayList(Gtk.Box):
 
     # song cache daemon
     def switch_caching_daemon(self, *args):
-        if self.cache_enabled is False:
+        if not self.cache_enabled:
             self.cache_enabled = True
             self.button_start.set_label(_('Stop Cache Service'))
             self.do_cache_song_pool()
@@ -435,17 +519,19 @@ class PlayList(Gtk.Box):
                 pass
             Gdk.Window.process_all_updates()
 
-        def _on_can_play(widget, song_path, status, error=None):
-            if status == 'OK':
-                return
-
-            # stop cache service when error occurs
+        def _on_disk_error(widget, song_path, eror=None):
             self.cache_enabled = False
-            if status == 'URLError':
-                Widgets.network_error(
-                        self.app.window, _('Failed to cache song'))
-            elif status == 'FileNotFoundError':
-                Widgets.filesystem_error(self.app.window, song_path)
+            GLib.idle_add(
+                    Widgets.filesystem_error,
+                    self.app.window,
+                    song_path)
+
+        def _on_network_error(widget, song_link, error=None):
+            self.cache_enabled = False
+            GLib.idle_add(
+                    Widgets.network_error,
+                    self.app.window,
+                    _('Failed to cache song'))
 
         def _on_downloaded(widget, song_path, error=None):
             if song_path:
@@ -472,8 +558,9 @@ class PlayList(Gtk.Box):
         song = Widgets.song_row_to_dict(liststore[path], start=0)
         print('will download:', song)
         self.cache_job = Net.AsyncSong(self.app)
-        self.cache_job.connect('can-play', _on_can_play)
         self.cache_job.connect('downloaded', _on_downloaded)
+        self.cache_job.connect('disk-error', _on_disk_error)
+        self.cache_job.connect('network-error', _on_network_error)
         self.cache_job.get_song(song)
 
     # Others
@@ -601,26 +688,6 @@ class PlayList(Gtk.Box):
         model.remove(_iter)
 
     def on_export_playlist_button_clicked(self, button):
-        def do_export(button):
-            num_songs = len(liststore)
-            export_dir = folder_chooser.get_filename()
-            export_lrc = with_lrc.get_active()
-            for i, item in enumerate(liststore):
-                song = Widgets.song_row_to_dict(item, start=0)
-                song_link, song_path = Net.get_song_link(
-                        song, self.app.conf)
-                if song_link is not True:
-                    continue
-                shutil.copy(song_path, export_dir)
-
-                if export_lrc:
-                    (lrc_path, lrc_cached) = Net.get_lrc_path(song)
-                    if lrc_cached:
-                        shutil.copy(lrc_path, export_dir)
-                export_prog.set_fraction(i / num_songs)
-                Gdk.Window.process_all_updates()
-            dialog.destroy()
-
         selection = self.treeview_left.get_selection()
         model, _iter = selection.get_selected()
         if not _iter:
@@ -630,46 +697,9 @@ class PlayList(Gtk.Box):
         disname, list_name, editable, tooltip = model[path]
         liststore = self.tabs[list_name].liststore
 
-        dialog = Gtk.Dialog(
-                _('Export Songs'), self.app.window, Gtk.DialogFlags.MODAL,
-                (Gtk.STOCK_CLOSE, Gtk.ResponseType.OK,))
-        box = dialog.get_content_area()
-        box.set_size_request(600, 320)
-        box.set_border_width(5)
-
-        folder_label = Widgets.BoldLabel(_('Choose export folder'))
-        box.pack_start(folder_label, False, True, 0)
-
-        folder_chooser = Widgets.FolderChooser(self.app.window)
-        folder_chooser.props.margin_left = 20
-        box.pack_start(folder_chooser, False, True, 0)
-
-        with_lrc = Gtk.CheckButton(_('With lyrics'))
-        with_lrc.set_tooltip_text(_('Export lyrics to the same folder'))
-        with_lrc.props.margin_top = 20
-        box.pack_start(with_lrc, False, False, 0)
-
-        export_box = Gtk.Box(spacing=5)
-        export_box.props.margin_top = 20
-        box.pack_start(export_box, False, True, 0)
-
-        export_prog = Gtk.ProgressBar()
-        export_box.pack_start(export_prog, True, True, 0)
-
-        export_btn = Gtk.Button(_('Export'))
-        export_btn.connect('clicked', do_export)
-        export_box.pack_start(export_btn, False, False, 0)
-
-        infobar = Gtk.InfoBar()
-        infobar.props.margin_top = 20
-        box.pack_start(infobar, False, True, 0)
-        info_content = infobar.get_content_area()
-        info_label = Gtk.Label(_('Only cached songs will be exported'))
-        info_content.pack_start(info_label, False, False, 0)
-
-        box.show_all()
-        dialog.run()
-        dialog.destroy()
+        export_dialog = ExportDialog(self, liststore)
+        export_dialog.run()
+        export_dialog.destroy()
 
     def advise_new_playlist_name(self, disname):
         self.playlist_advice_disname = disname
