@@ -12,8 +12,6 @@ from gi.repository import Gst
 from gi.repository import GstVideo
 from gi.repository import Gtk
 
-# Or init threads in another place.
-GLib.threads_init()
 # init Gst so that play works ok.
 Gst.init(None)
 GST_LOWER_THAN_1 = (Gst.version()[0] < 1)
