@@ -83,8 +83,8 @@ class Themes(Gtk.Box):
         if not nodes:
             print('Failed to get nodes, do something!')
             return
-        for i, node in enumerate(nodes):
-            self.liststore_main.append([
+        for node in nodes:
+            tree_iter = self.liststore_main.append([
                 self.app.theme['anonymous'],
                 Widgets.unescape(node['name']),
                 int(node['nid']),
@@ -92,7 +92,7 @@ class Themes(Gtk.Box):
                 Widgets.set_tooltip(node['name'], node['info']),
                 ])
             Net.update_liststore_image(
-                    self.liststore_main, i, 0, node['pic'])
+                    self.liststore_main, tree_iter, 0, node['pic'])
 
     def on_iconview_main_item_activated(self, iconview, path):
         model = iconview.get_model()
@@ -116,9 +116,8 @@ class Themes(Gtk.Box):
                 self.curr_sub_id, self.nodes_page)
         if not nodes:
             return
-        i = len(self.liststore_sub)
         for node in nodes:
-            self.liststore_sub.append([
+            tree_iter = self.liststore_sub.append([
                 self.app.theme['anonymous'],
                 Widgets.unescape(node['name']),
                 int(node['sourceid']),
@@ -127,8 +126,7 @@ class Themes(Gtk.Box):
                     node['name'], node['tips']),
                 ])
             Net.update_liststore_image(
-                    self.liststore_sub, i, 0, node['pic'])
-            i += 1
+                    self.liststore_sub, tree_iter, 0, node['pic'])
 
     def on_iconview_sub_item_activated(self, iconview, path):
         model = iconview.get_model()
